@@ -6,8 +6,7 @@
 ;; URL: http://github.com/s-kostyaev/ellama
 ;; Keywords: help local tools
 ;; Package-Requires: ((emacs "28.1") (llm "0.24.0") (plz "0.8") (transient "0.7") (compat "29.1") (yaml "1.2.3"))
-;; Package-Version: 1.11.1
-;; Package-Revision: d9ee61b4da10
+;; Version: 1.12.0
 ;; SPDX-License-Identifier: GPL-3.0-or-later
 ;; Created: 8th Oct 2023
 
@@ -1504,6 +1503,8 @@ in.  Default value is (current-buffer).
 
 :point POINT -- POINT is the point in buffer to insert ellama reply at.
 
+:tools LIST -- LIST of enabled in the current session tools.
+
 :filter FILTER -- FILTER is a function that's applied to (partial) response
 strings before they're inserted into the BUFFER.
 
@@ -1550,6 +1551,12 @@ failure (with BUFFER current).
 	 (prompt-with-ctx (ellama-context-prompt-with-context prompt))
 	 (system (or (plist-get args :system)
 		     (ellama-get-system-message)))
+	 (session-tools (and session
+                             (ellama-session-extra session)
+                             (plist-get (ellama-session-extra session) :tools)))
+	 (tools (or session-tools
+		    (plist-get args :tools)
+		    ellama-tools-enabled))
 	 (llm-prompt (if session
 			 (if (llm-chat-prompt-p (ellama-session-prompt session))
 			     (progn
@@ -1557,7 +1564,7 @@ failure (with BUFFER current).
 				(ellama-session-prompt session)
 				prompt-with-ctx)
 			       (setf (llm-chat-prompt-tools (ellama-session-prompt session))
-				     ellama-tools-enabled)
+				     tools)
 			       (when system
 				 (llm-chat-prompt-append-response
 				  (ellama-session-prompt session)
@@ -1565,9 +1572,9 @@ failure (with BUFFER current).
 			       (ellama-session-prompt session))
 			   (setf (ellama-session-prompt session)
 				 (llm-make-chat-prompt prompt-with-ctx :context system
-						       :tools ellama-tools-enabled)))
+						       :tools tools)))
 		       (llm-make-chat-prompt prompt-with-ctx :context system
-					     :tools ellama-tools-enabled))))
+					     :tools tools))))
     (with-current-buffer reasoning-buffer
       (org-mode))
     (with-current-buffer buffer
